@@ -48,7 +48,7 @@ use vars qw/@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS
             $TIMEOUT @MODULES %MODULES %METHODS $AUTOLOAD
             @CURRENCY_RATES_MODULES $USE_EXPERIMENTAL_UA/;
 
-our $VERSION = '1.66_02'; # TRIAL VERSION
+our $VERSION = '1.67_01'; # TRIAL VERSION
 
 @CURRENCY_RATES_MODULES = qw/
     AlphaVantage
@@ -70,7 +70,6 @@ our $VERSION = '1.66_02'; # TRIAL VERSION
     Bourso
     BVB
     CSE
-    CMBChina
     Comdirect
     Consorsbank
     Currencies
@@ -1715,7 +1714,6 @@ http://www.gnucash.org/
   Finance::Quote::CSE,
   Finance::Quote::Comdirect,
   Finance::Quote::Consorsbank,
-  Finance::Quote::CMBChina,
   Finance::Quote::Currencies,
   Finance::Quote::Deka,
   Finance::Quote::FinanceAPI,
